//
// Created by gunner on 17/05/21.
//
#include "ModelOBJ.h"
#include <string>
#include <cstring>
#include <vector>
#include <math.h>
#include <algorithm>
#include <iostream>
#include <iomanip>
#include <fstream>

using namespace std;
using std::vector;

ModelOBJ::ModelOBJ()= default;

unsigned int mLUT[3][256];

//Method to read and obtain the data from de obj

int ModelOBJ::import(string pathFile){
    char * path = new char [pathFile.length()];
    strcpy(path,pathFile.c_str());
    FILE * file = fopen(path, "r");
    char lineHeader[128];
    if( file == NULL ){
        printf("Impossible to open the file !\n");
        return false;
    }
    while(fscanf(file, "%s", lineHeader) != EOF){
        if ( strcmp( lineHeader, "v" ) == 0 ){
            UniqueVertex vertice;
            fscanf(file, "%f %f %f\n", &vertice.vertex.x, &vertice.vertex.y, &vertice.vertex.z);
            if (vertice.vertex.x < minCoordX){
                minCoordX = vertice.vertex.x;
            }
            if (vertice.vertex.y < minCoordY){
                minCoordY = vertice.vertex.y;
            }
            if (vertice.vertex.z < minCoordZ){
                minCoordZ = vertice.vertex.z;
            }
            if (vertice.vertex.x > maxCoordX){
                maxCoordX = vertice.vertex.x;
            }
            if (vertice.vertex.y > maxCoordY){
                maxCoordY = vertice.vertex.y;
            }
            if (vertice.vertex.z > maxCoordZ){
                maxCoordZ = vertice.vertex.z;
            }
            coordsX.push_back(vertice.vertex.x);
            coordsY.push_back(vertice.vertex.y);
            coordsZ.push_back(vertice.vertex.z);
            uniqueVertices.push_back(vertice);
            numVertices++;
        }else if ( strcmp( lineHeader, "vt" ) == 0 ){
            numTextures++;
        }else if ( strcmp( lineHeader, "vn" ) == 0 ){
            numNormals++;
        }else if ( strcmp( lineHeader, "f" ) == 0 ) {
            numGeometries++;
        }
    }

    fclose(file);
    Assimp::Importer importer;
    const aiScene* scene = importer.ReadFile(pathFile, 0/*aiProcess_JoinIdenticalVertices | aiProcess_SortByPType*/);
    if( !scene) {
        fprintf( stderr, "%s", importer.GetErrorString());
        getchar();
        exit(1);
    }
    clock_t start = clock();
    buildMesh(scene);
    clock_t mesht = clock();
    double timem = double(mesht - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"Mesh: "<< std::setprecision(12) << timem <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
    maxMinCoord();
    /**
    clock_t maxmin = clock();
    double timemaxmi = double(maxmin - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"MaxminC: "<< std::setprecision(12) << timemaxmi <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
    kMeansClustering();
    clock_t kM = clock();
    double timeKM = double(kM - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"Kmeans: "<< std::setprecision(12) << timeKM <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
    //precomputingLUT();
    clusteringMortonCodes(uniqueVertices);
    clock_t mC = clock();
    double timeMC = double(mC - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"MortonC: "<< std::setprecision(12) << timeMC <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
    //histogramFrequency();
    histogramFrequencyX();
    histogramFrequencyY();
    histogramFrequencyZ();
    clock_t hisF = clock();
    double timeHF = double(hisF - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"Histograms: "<< std::setprecision(12) << timeHF <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
    espacialHistogramsX();
    espacialHistogramsY();
    espacialHistogramsZ();
    clock_t esH = clock();
    double timeEH = double(esH - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"Espacial histograms: "<< std::setprecision(12) << timeEH <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
    //centralTendency();
    //standardDeviation();
    centralTendencyCoordsX();
    standardDeviationCoordsX();
    centralTendencyCoordsY();
    standardDeviationCoordsY();
    centralTendencyCoordsZ();
    standardDeviationCoordsZ();
    clock_t cT = clock();
    double timecT = double(cT - start)/CLOCKS_PER_SEC;
    std::cout<<"----------------------------------"<<std::endl;
    std::cout<<"central tendency and Standar Derivation: "<< std::setprecision(12) << timecT <<std::endl;
    std::cout<<"----------------------------------"<<std::endl;
     **/
    return true;
}

//Method to obtain the meshes from the scene

void ModelOBJ::buildMesh(const aiScene* scene){
    numMeshes = scene->mNumMeshes;
    for (int i = 0; i < scene->mNumMeshes; i++)
    {
        const aiMesh* mesh = scene->mMeshes[i];
        loadMesh(mesh);
        meshes.push_back(mesh);
    }
    centroid();
}


void ModelOBJ::loadMesh(const aiMesh *mesh) {
    Mesh newMesh;
    newMesh.coordsV.reserve(mesh->mNumVertices);
    newMesh.textureC.reserve(mesh->mNumVertices);
    newMesh.normalV.reserve(mesh->mNumVertices);
    newMesh.indices.reserve(3 * mesh->mNumFaces);
    for (unsigned int i = 0; i < mesh->mNumVertices; i++) {
        // Obtener la posicion de cada vertice
        Vertex coords;
        aiVector3D pos = mesh->mVertices[i];
        coords.vertex.x = pos.x;
        coords.vertex.y = pos.y;
        coords.vertex.z = pos.z;
        newMesh.coordsV.push_back(coords);

        // Obtener las coordenadas de textura
        if (mesh->HasTextureCoords(0)) {
            TexturesV textV;
            aiVector3D UVW = mesh->mTextureCoords[0][i];
            textV.textures.x = UVW.x;
            textV.textures.y = UVW.y;
            newMesh.textureC.push_back(textV);
        }

        // Obtener los vectores normales
        if (mesh->HasNormals()) {
            NormalsV normalV;
            aiVector3D n = mesh->mNormals[i];
            normalV.normals.x = n.x;
            normalV.normals.y = n.y;
            normalV.normals.z = n.z;
            newMesh.normalV.push_back(normalV);
        }
    }
    // Obtener los indices
    for (unsigned int i = 0; i < mesh->mNumFaces; i++) {
        newMesh.indices.push_back(mesh->mFaces[i].mIndices[0]);
        newMesh.indices.push_back(mesh->mFaces[i].mIndices[1]);
        newMesh.indices.push_back(mesh->mFaces[i].mIndices[2]);
    }
    listMeshes.push_back(newMesh);
}


//Method for obtain de centroid of every face in a mesh

void ModelOBJ::centroid(){
    for (int i = 0; i < numMeshes; i++) {
        const aiMesh* malla = meshes[i];
        vector<glm::vec3> centroidsInMesh;
        for (int j = 0; j < malla->mNumFaces; j++) {
            glm::vec3 centroid;
            glm::vec3 vertice1;
            glm::vec3 vertice2;
            glm::vec3 vertice3;
            vertice1.x = malla->mVertices[listMeshes[i].indices[j*3]].x;
            vertice1.y = malla->mVertices[listMeshes[i].indices[j*3]].y;
            vertice1.z = malla->mVertices[listMeshes[i].indices[j*3]].z;
            vertice2.x = malla->mVertices[listMeshes[i].indices[(j*3)+1]].x;
            vertice2.y = malla->mVertices[listMeshes[i].indices[(j*3)+1]].y;
            vertice2.z = malla->mVertices[listMeshes[i].indices[(j*3)+1]].z;
            vertice3.x = malla->mVertices[listMeshes[i].indices[(j*3)+2]].x;
            vertice3.y = malla->mVertices[listMeshes[i].indices[(j*3)+2]].y;
            vertice3.z = malla->mVertices[listMeshes[i].indices[(j*3)+2]].z;
            centroid.x = (vertice1.x+vertice2.x+vertice3.x)/3;
            centroid.y = (vertice1.y+vertice2.y+vertice3.y)/3;
            centroid.z = (vertice1.z+vertice2.z+vertice3.z)/3;
            centroids.push_back(centroid);
            centroidsInMesh.push_back(centroid);
        }
        centroidsByMesh.push_back(centroidsInMesh);
    }
}

uint32_t ModelOBJ::codeMortonCode(uint32_t x, uint32_t y, uint32_t z){
    x = (x | (x << 16)) & 0x030000FF;
    x = (x | (x <<  8)) & 0x0300F00F;
    x = (x | (x <<  4)) & 0x030C30C3;
    x = (x | (x <<  2)) & 0x09249249;
    //std::cout<< "x: " << x << std::endl;
    y = (y | (y << 16)) & 0x030000FF;
    y = (y | (y <<  8)) & 0x0300F00F;
    y = (y | (y <<  4)) & 0x030C30C3;
    y = (y | (y <<  2)) & 0x09249249;
    //std::cout<< "y: " << y << std::endl;
    z = (z | (z << 16)) & 0x030000FF;
    z = (z | (z <<  8)) & 0x0300F00F;
    z = (z | (z <<  4)) & 0x030C30C3;
    z = (z | (z <<  2)) & 0x09249249;
    //std::cout<< "z: " << z << std::endl;
    //uint32_t res = (x | (y << 1) | (z << 2));
    //std::cout<< "resultado: " << res << std::endl;
    return (x | (y << 1) | (z << 2));
    //return separateBy2(x) | separateBy2(y << 1) | separateBy2(z << 2);
}

ModelOBJ::Vertex ModelOBJ::decodeMortonCode(uint32_t mortonC){
    /*Vertex v;
    v.vertex.x = compactBy2(mortonC);
    v.vertex.y = compactBy2(mortonC >> 1);
    v.vertex.z = compactBy2(mortonC >> 2);
    std::cout<< "vertice: " << v.vertex.x << "," << v.vertex.y << "," << v.vertex.z << std::endl;
    return v;*/
    uint32_t x = 0,y = 0,z = 0;
    x = mortonC &        0x09249249;
    y = (mortonC >> 1) & 0x09249249;
    z = (mortonC >> 2) & 0x09249249;

    x = ((x >> 2) | x) & 0x030C30C3;
    x = ((x >> 4) | x) & 0x0300F00F;
    x = ((x >> 8) | x) & 0x030000FF;
    x = ((x >>16) | x) & 0x000003FF;

    y = ((y >> 2) | y) & 0x030C30C3;
    y = ((y >> 4) | y) & 0x0300F00F;
    y = ((y >> 8) | y) & 0x030000FF;
    y = ((y >>16) | y) & 0x000003FF;

    z = ((z >> 2) | z) & 0x030C30C3;
    z = ((z >> 4) | z) & 0x0300F00F;
    z = ((z >> 8) | z) & 0x030000FF;
    z = ((z >>16) | z) & 0x000003FF;

    Vertex v;
    float xf = x;
    float yf = y;
    float zf = z;
    v.vertex.x = xf/1023;
    v.vertex.y = yf/1023;
    v.vertex.z = zf/1023;
    return v;
}

ModelOBJ::UniqueVertex ModelOBJ::normalize(UniqueVertex v) {
    UniqueVertex normalVertex;
    //std::cout<< "Divisiones: " << v.x-minCoordX << "," << (maxCoordX-minCoordX) << "," << (v.y-minCoordY) << "," << (maxCoordY-minCoordY) << "," << (v.z-minCoordZ)/(maxCoordZ-minCoordZ) << std::endl;
    normalVertex.vertex.x = (v.vertex.x-minCoordX)/(maxCoordX-minCoordX);
    normalVertex.vertex.y = (v.vertex.y-minCoordY)/(maxCoordY-minCoordY);
    normalVertex.vertex.z = (v.vertex.z-minCoordZ)/(maxCoordZ-minCoordZ);
    normalVertex.vertex.x = normalVertex.vertex.x * 1023;
    normalVertex.vertex.y = normalVertex.vertex.y * 1023;
    normalVertex.vertex.z = normalVertex.vertex.z * 1023;
    //std::cout << "vertice al normalizar: " << std::setprecision(9) << normalVertex.x << "," << normalVertex.y << "," << normalVertex.z << std::endl;
    return normalVertex;
}

void ModelOBJ::clusteringMortonCodes(vector<UniqueVertex> v) {
    for (UniqueVertex vertex : v) {
        //std::cout<< "vertice: " << vertex.x << "," << vertex.y << "," << vertex.z << std::endl;
        if (vertex.vertex.x < 0 || vertex.vertex.y < 0 || vertex.vertex.z < 0){
            UniqueVertex normalV = normalize(vertex);
            //std::cout<< "vertice normalizado: " << std::setprecision(9) << normalV.x << "," << normalV.y << "," << normalV.z << std::endl;
            uint32_t mc = codeMortonCode(normalV.vertex.x,normalV.vertex.y,normalV.vertex.z);
            //std::cout<< "mc: " << mc << std::endl;
            mortonCodes.push_back(mc);
        }else{
            uint32_t mc = codeMortonCode(vertex.vertex.x*1023,vertex.vertex.y*1023,vertex.vertex.z*1023);
            //std::cout<< "mc: " << mc << std::endl;
            mortonCodes.push_back(mc);
        }
    }
    std::sort(mortonCodes.begin(), mortonCodes.end());
    int k = 1 + log2(numVertices);
    int elemClusters = numVertices/k;
    int clusters[k][elemClusters];
    int i = 0;
    int j = 0;
    for (int l = 0; l < numVertices; ++l) {
        if (i >= elemClusters){
            i = 0;
            j++;
        }
        clusters[j][i] = mortonCodes[l];
        i++;
    }
    ofstream myfile;
    myfile.open("outputMortonCodes.csv");
    myfile << "x,y,z,c" << endl;
    for (int i = 0; i < k;i++) {
        for (int z = 0; z < elemClusters; z++) {
            //std::cout<< "Morton Code: " << clusters[i][z] << "," << i << "," << z <<std::endl;
            Vertex csv = decodeMortonCode(clusters[i][z]);
            myfile << std::setprecision(3) << csv.vertex.x << "," << csv.vertex.y << "," << csv.vertex.z << "," << i << endl;
        }
    }
    myfile.close();
}

void ModelOBJ::kMeansClustering() {
    std::vector<glm::vec3> centroidsKM;
    bool sameDist = false;
    float k = 1 + log2(numVertices);
    int epoch = 500;
    int cost = 0;
    int e = 0;
    srand(time(0));
    for (float i = 0; i < k; i++) {
        glm::vec3 tempC = centroids.at(rand()%centroids.size());
        //cout << tempC.x << "," << tempC.y << "," << tempC.z << endl;
        centroidsKM.push_back(tempC);
    }
    for (int j = 0; j < epoch; j++) {
        cout << e << endl;
        e++;
        if(sameDist == true) break;
        for(auto c = begin(centroidsKM);c != end(centroidsKM);c++){
            int clusterID = c - begin(centroidsKM);
            for (auto v = begin(uniqueVertices);v != end(uniqueVertices);v++) {
                UniqueVertex temp = *v;
                double distanceCentroid = sqrt(pow(v->vertex.x-c->x,2)+pow(v->vertex.y-c->y,2)
                                               +pow(v->vertex.z-c->z,2));
                cost += distanceCentroid;

                if (distanceCentroid < temp.minDistanceC){
                    temp.minDistanceC = distanceCentroid;
                    temp.clusterId = clusterID;
                    clusterinepoch.push_back(clusterID);
                    //cout << clusterID << "," << distanceCentroid << "," << temp.vertex.x << "," << temp.vertex.y << "," << temp.vertex.z << endl;
                }
                if (distanceCentroid == temp.minDistanceC && !(find(clusterinepoch.begin(), clusterinepoch.end(), clusterID) != clusterinepoch.end())){
                    temp.minDistanceC = distanceCentroid;
                    temp.clusterId = clusterID;
                    clusterinepoch.push_back(clusterID);
                }
                *v = temp;
            }
        }
        cost = cost/numGeometries;
        std::vector<int> numVertexinCluster;
        std::vector<double> sumX,sumY,sumZ;
        for (int m = 0; m < k; m++) {
            numVertexinCluster.push_back(0);
            sumX.push_back(0.0);
            sumY.push_back(0.0);
            sumZ.push_back(0.0);
        }
        for(auto p = begin(uniqueVertices);p != end(uniqueVertices);p++){
            int cID = p->clusterId;
            numVertexinCluster[cID] += 1;
            sumX[cID] += p->vertex.x;
            sumY[cID] += p->vertex.y;
            sumZ[cID] += p->vertex.z;
            //cout << sumX[cID] << ", "<< sumY[cID] << ", "<< sumZ[cID] << endl;
            p->minDistanceC = __DBL_MAX__;
        }
        for(auto e = begin(centroidsKM);e != end(centroidsKM);e++){
            int cID = e - begin(centroidsKM);
            glm::vec3 newCentroid;
            newCentroid.x = sumX[cID]/numVertexinCluster[cID];
            newCentroid.y = sumY[cID]/numVertexinCluster[cID];
            newCentroid.z = sumZ[cID]/numVertexinCluster[cID];
            if (newCentroid.x == e->x & newCentroid.y == e->y & newCentroid.z == e->z){
                sameDist = true;
                break;
            }
            e->x = newCentroid.x;
            e->y = newCentroid.y;
            e->z = newCentroid.z;
            //cout << e->x << ", "<< e->y << ", "<< e->z << ", epoch: " << j << endl;
        }
    }
    std::ofstream myfile;
    myfile.open("outputKmeans.csv");
    myfile << "x,y,z,c" << endl;
    for (vector<UniqueVertex>::iterator it = uniqueVertices.begin(); it != uniqueVertices.end();
         it++) {
        myfile << std::setprecision(3) << it->vertex.x << "," << it->vertex.y << "," << it->vertex.z << "," << it->clusterId << endl;
    }
    myfile.close();
    myfile.open("centroidsKmeans.csv");
    myfile << "x,y,z,c" << endl;
    int nc = 0;
    for (vector<glm::vec3>::iterator it = centroidsKM.begin();it != centroidsKM.end();it++) {
        myfile << it->x << "," << it->y << "," << it->z << "," << nc << endl;
        nc++;
    }
    myfile.close();
}

void ModelOBJ::cTandsDKmeas(){
    vector<vector<float>> distancesByCluster;
    float k = 1 + log2(numVertices);
    for (int i = 0; i < k; i++) {
        vector<float> distancesinCluster;
        distancesByCluster.push_back(distancesinCluster);
        for (UniqueVertex v:uniqueVertices) {
            if (v.clusterId == i) distancesByCluster[i].push_back(v.minDistanceC);
        }
    }
    for(vector<float> vec:distancesByCluster){
        float average = 0;
        float middleCluster = 0;
        float sDerivationCluster = 0;
        for (float dis:vec) {
            average += dis;
        }
        average = average/vec.size();
        averageByCluster.push_back(average);
        if((vec.size()%2) == 1){
            middleCluster = vec[vec.size()/2];
            middleByCluster.push_back(middleCluster);
        } else{
            middleCluster = vec[vec.size()/2] + vec[(vec.size()/2)-1]/2;
            middleByCluster.push_back(middleCluster);
        }
        for (float dis:vec) {
            sDerivationCluster += pow(dis-average,2);
        }
        sDerivationCluster = sDerivationCluster/vec.size();
        sDerivationByCluster.push_back(sDerivationCluster);
    }
}

void ModelOBJ::frequenciesX() {
    float range = maxCoordX - minCoordX;
    float k = 1 + log2(coordsX.size());
    float h = range/k;
    float aux = minCoordX;
    int intervals = 1;
    while (aux <= maxCoordX+h){
        aux += h;
        intervals++;
    }
    float temp = minCoordX;
    for (int e = 0; e < intervals; e++) {
        glm::vec2 interval;
        interval.x = temp;
        interval.y = 0;
        temp += h;
        frequenciesCX.push_back(interval);
    }
    for (float i : coordsX) {
        for (int j = 0; j < frequenciesCX.size(); ++j) {
            if (j == 0){
                if (i <= frequenciesCX[j].x){
                    frequenciesCX[j].y++;
                    break;
                }
            } else{
                if (i <= frequenciesCX[j].x && i > frequenciesCX[j-1].x){
                    frequenciesCX[j].y++;
                    break;
                }
            }
        }
    }
}

void ModelOBJ::frequenciesY() {
    float range = maxCoordY - minCoordY;
    float k = 1 + log2(coordsY.size());
    float h = range/k;
    float aux = minCoordY;
    int intervals = 1;
    while (aux <= maxCoordY+h){
        aux += h;
        intervals++;
    }
    float temp = minCoordY;
    for (int i = 0; i < intervals; i++) {
        glm::vec2 interval;
        interval.x = temp;
        interval.y = 0;
        temp += h;
        frequenciesCY.push_back(interval);
    }
    for (float i : coordsY) {
        for (int j = 0; j < frequenciesCY.size(); ++j) {
            if (j == 0){
                if (i <= frequenciesCY[j].x){
                    frequenciesCY[j].y++;
                    break;
                }
            } else{
                if (i <= frequenciesCY[j].x && i > frequenciesCY[j-1].x){
                    frequenciesCY[j].y++;
                    break;
                }
            }
        }
    }
}

void ModelOBJ::frequenciesZ() {
    float range = maxCoordZ - minCoordZ;
    float k = 1 + log2(coordsZ.size());
    float h = range/k;
    float aux = minCoordZ;
    int intervals = 1;
    while (aux <= maxCoordY+h){
        aux += h;
        intervals++;
    }
    float temp = minCoordZ;
    for (int e = 0; e < intervals; e++) {
        glm::vec2 interval;
        interval.x = temp;
        interval.y = 0;
        temp += h;
        frequenciesCZ.push_back(interval);
    }
    for (float i : coordsZ) {
        for (int j = 0; j < frequenciesCZ.size(); ++j) {
            if (j == 0){
                if (i <= frequenciesCZ[j].x){
                    frequenciesCZ[j].y++;
                    break;
                }
            } else{
                if (i <= frequenciesCZ[j].x && i > frequenciesCZ[j-1].x){
                    frequenciesCZ[j].y++;
                    break;
                }
            }
        }
    }
}

void ModelOBJ::frequenciesEX(){
    for (int i = 0; i < numMeshes; i++) {
        float range = maxminCoords[i][0] - maxminCoords[i][1];
        if (range == 0) break;
        float k = 1 + log2(uniqueVertices.size());
        float h = range/k;
        float aux = maxminCoords[i][1];
        int intervals = 1;
        while (aux <= maxminCoords[i][0]+h){
            aux += h;
            intervals++;
        }
        float temp = maxminCoords[i][1];
        for (int e = 0; e < intervals; e++) {
            glm::vec2 interval;
            interval.x = temp;
            interval.y = 0;
            temp += h;
            espacialFreqEX.push_back(interval);
        }
        for (auto & uniqueVertice : uniqueVertices) {
            for (int z = 0; z < espacialFreqEX.size(); z++) {
                if (z == 0){
                    if (uniqueVertice.vertex.x <= espacialFreqEX[z].x){
                        espacialFreqEX[z].y++;
                        break;
                    }
                }else{
                    if (uniqueVertice.vertex.x <= espacialFreqEX[z].x && uniqueVertice.vertex.x > espacialFreqEX[z-1].x){
                        espacialFreqEX[z].y++;
                        break;
                    }
                }
            }
        }
    }
}

void ModelOBJ::frequenciesEY(){
    for (int i = 0; i < numMeshes; i++) {
        float range = maxminCoords[i][2] - maxminCoords[i][3];
        if (range == 0) break;
        float k = 1 + log2(uniqueVertices.size());
        float h = range/k;
        float aux = maxminCoords[i][3];
        int intervals = 1;
        while (aux <= maxminCoords[i][2]+h){
            aux += h;
            intervals++;
        }
        float temp = maxminCoords[i][3];
        for (int e = 0; e < intervals; e++) {
            glm::vec2 interval;
            interval.x = temp;
            interval.y = 0;
            temp += h;
            espacialFreqEY.push_back(interval);
        }
        for (auto & uniqueVertice : uniqueVertices) {
            for (int z = 0; z < espacialFreqEY.size(); z++) {
                if (z == 0){
                    if (uniqueVertice.vertex.x <= espacialFreqEY[z].x){
                        espacialFreqEY[z].y++;
                        break;
                    }
                }else{
                    if (uniqueVertice.vertex.x <= espacialFreqEY[z].x && uniqueVertice.vertex.x > espacialFreqEY[z-1].x){
                        espacialFreqEY[z].y++;
                        break;
                    }
                }
            }
        }
    }
}

void ModelOBJ::frequenciesEZ(){
    for (int i = 0; i < numMeshes; i++) {
        float range = maxminCoords[i][4] - maxminCoords[i][5];
        if (range == 0) break;
        float k = 1 + log2(uniqueVertices.size());
        float h = range/k;
        float aux = maxminCoords[i][5];
        int intervals = 1;
        while (aux <= maxminCoords[i][4]+h){
            aux += h;
            intervals++;
        }
        float temp = maxminCoords[i][5];
        for (int e = 0; e < intervals; e++) {
            glm::vec2 interval;
            interval.x = temp;
            interval.y = 0;
            temp += h;
            espacialFreqEZ.push_back(interval);
        }
        for (auto & uniqueVertice : uniqueVertices) {
            for (int z = 0; z < espacialFreqEZ.size(); z++) {
                if (z == 0){
                    if (uniqueVertice.vertex.x <= espacialFreqEZ[z].x){
                        espacialFreqEZ[z].y++;
                        break;
                    }
                }else{
                    if (uniqueVertice.vertex.x <= espacialFreqEZ[z].x && uniqueVertice.vertex.x > espacialFreqEZ[z-1].x){
                        espacialFreqEZ[z].y++;
                        break;
                    }
                }
            }
        }
    }
}

void ModelOBJ::maxMinCoord(){
    for (int i = 0; i < numMeshes; i++) {
        const aiMesh* malla = meshes[i];
        std::vector<float> maxmins;
        float maxX=0,minX=0,maxY=0,minY=0,maxZ=0,minZ=0;
        for (int j = 0; j < malla->mNumVertices; j++) {
            if (j == 0){
                minX = malla->mVertices[j].x;
                minY = malla->mVertices[j].y;
                minZ = malla->mVertices[j].z;
            }
            if (malla->mVertices[j].x > maxX){
                maxX = malla->mVertices[j].x;
            }
            if (malla->mVertices[j].y > maxY){
                maxY = malla->mVertices[j].y;
            }
            if (malla->mVertices[j].z > maxZ){
                maxZ = malla->mVertices[j].z;
            }
            if (malla->mVertices[j].x < minX){
                minX = malla->mVertices[j].x;
            }
            if (malla->mVertices[j].y < minY){
                minY = malla->mVertices[j].y;
            }
            if (malla->mVertices[j].z < minZ){
                minZ = malla->mVertices[j].z;
            }
        }
        maxmins.push_back(maxX);
        maxmins.push_back(minX);
        maxmins.push_back(maxY);
        maxmins.push_back(minY);
        maxmins.push_back(maxZ);
        maxmins.push_back(minZ);
        maxminCoords.push_back(maxmins);
    }
}

void ModelOBJ::histogramFrequencyX() {
    frequenciesX();
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"\t          Histogram Frequency X"<<std::endl;
    std::cout<<"--------------------------------------------------"<<std::endl;
    for (auto & i : frequenciesCX) {
        std::cout<< "Interval: " << std::setprecision(9) << i.x << "   Frecuency: " << i.y << std::endl;
    }
    std::cout<< "Min: " << std::setprecision(9) << minCoordX << "   Max: " << maxCoordX << std::endl;
}

void ModelOBJ::histogramFrequencyY() {
    frequenciesY();
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"\t          Histogram Frequency Y"<<std::endl;
    std::cout<<"--------------------------------------------------"<<std::endl;
    for (auto & i : frequenciesCY) {
        std::cout<< "Interval: " << std::setprecision(9) << i.x << "   Frecuency: " << i.y << std::endl;
    }
    std::cout<< "Min: " << std::setprecision(9) << minCoordY << "   Max: " << maxCoordY << std::endl;
}

void ModelOBJ::histogramFrequencyZ() {
    frequenciesZ();
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"\t          Histogram Frequency Z"<<std::endl;
    std::cout<<"--------------------------------------------------"<<std::endl;
    for (auto & i : frequenciesCZ) {
        std::cout<< "Interval: " << std::setprecision(9) << i.x << "   Frecuency: " << i.y << std::endl;
    }
    std::cout<< "Min: " << std::setprecision(9) << minCoordZ << "   Max: " << maxCoordZ << std::endl;
}

void ModelOBJ::espacialHistogramsX(){
    frequenciesEX();
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"\t          Espacial Histogram Frequency X "<<std::endl;
    std::cout<<"--------------------------------------------------"<<std::endl;
    for (auto & i : espacialFreqEX) {
        std::cout<< "Interval: " << std::setprecision(9) << i.x << "   Frecuency: " << i.y << std::endl;
    }
}

void ModelOBJ::espacialHistogramsY(){
    frequenciesEY();
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"\t          Espacial Histogram Frequency Y "<<std::endl;
    std::cout<<"--------------------------------------------------"<<std::endl;
    for (auto & i : espacialFreqEY) {
        std::cout<< "Interval: " << std::setprecision(9) << i.x << "   Frecuency: " << i.y << std::endl;
    }
}

void ModelOBJ::espacialHistogramsZ(){
    frequenciesEZ();
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"\t          Espacial Histogram Frequency Z "<<std::endl;
    std::cout<<"--------------------------------------------------"<<std::endl;
    for (auto & i : espacialFreqEZ) {
        std::cout<< "Interval: " << std::setprecision(9) << i.x << "   Frecuency: " << i.y << std::endl;
    }
}

//Method for obtain the average and middle of coordenates of X

void ModelOBJ::centralTendencyCoordsX(){
    sort(coordsX.begin(),coordsX.end());
    for(float coordX : coordsX) {
        averageX += coordX;
    }
    averageX = averageX/coordsX.size();
    if((coordsX.size()%2) == 1){
        middleX = coordsX[coordsX.size()/2];
    } else{
        middleX = coordsX[coordsX.size()/2] + coordsX[(coordsX.size()/2)-1]/2;
    }
}

//Method for obtain the Standard Deviation of coordenates of X, having the average of coordenates of X.

void ModelOBJ::standardDeviationCoordsX(){
    for (float coordX : coordsX) {
        sDeviationX += pow(coordX-averageX,2);
    }
    sDeviationX = sDeviationX/coordsX.size();
}

//Method for obtain the average and middle of coordenates of Y

void ModelOBJ::centralTendencyCoordsY(){
    sort(coordsY.begin(),coordsY.end());
    for(float coordY : coordsY) {
        averageY += coordY;
    }
    averageY = averageY/coordsY.size();
    if((coordsY.size()%2) == 1){
        middleY = coordsY[coordsY.size()/2];
    } else{
        middleY = coordsY[coordsY.size()/2] + coordsY[(coordsY.size()/2)-1]/2;
    }
}

//Method for obtain the Standard Deviation of coordenates of Y, having the average of coordenates of Y.

void ModelOBJ::standardDeviationCoordsY(){
    for (float coordY : coordsY) {
        sDeviationY += pow(coordY-averageY,2);
    }
    sDeviationY = sDeviationY/coordsY.size();
}

//Method for obtain the average and middle of coordenates of Z

void ModelOBJ::centralTendencyCoordsZ(){
    sort(coordsZ.begin(),coordsZ.end());
    for(float coordZ : coordsZ) {
        averageZ += coordZ;
    }
    averageZ = averageZ/coordsZ.size();
    if((coordsZ.size()%2) == 1){
        middleZ = coordsZ[coordsZ.size()/2];
    } else{
        middleZ = coordsZ[coordsZ.size()/2] + coordsZ[(coordsZ.size()/2)-1]/2;
    }
}

//Method for obtain the Standard Deviation of distances between the coordenates of Z, having the average of coordenates of Z.

void ModelOBJ::standardDeviationCoordsZ(){
    for (float coordZ : coordsZ) {
        sDeviationZ += pow(coordZ-averageZ,2);
    }
    sDeviationZ = sDeviationZ/coordsZ.size();
}

//Method for obtain the average and middle of distances between the centroids

void ModelOBJ::centralTendency(){
    sort(distances.begin(),distances.end());
    for(float distance : distances) {
        average += distance;
    }
    average = average/distances.size();
    if((distances.size()%2) == 1){
        middle = distances[distances.size()/2];
    } else{
        middle = distances[distances.size()/2] + distances[(distances.size()/2)-1]/2;
    }
}

//Method for obtain the Standard Deviation of distances between the centroids, having the average of distances.

void ModelOBJ::standardDeviation(){
    for (float distance : distances) {
        sDeviation += pow(distance-average,2);
    }
    sDeviation = sDeviation/distances.size();
}