//
// Created by gunner on 17/05/21.
//

#include <glm/glm.hpp>
#include <string>
#include <vector>
#include <assimp/Importer.hpp>
#include <assimp/scene.h>
#ifndef MODELOBJ_H
#define MODELOBJ_H
using std::vector;
using std::string;

class ModelOBJ
{
public:
    struct UniqueVertex{
        //float x = 0.0,y = 0.0,z = 0.0;
        glm::vec3 vertex;
        double minDistanceC = __DBL_MAX__;
        int clusterId = -1;
    };

    struct Vertex
    {
        glm::vec3 vertex;
    };

    struct TexturesV
    {
        glm::vec2 textures;
    };

    struct NormalsV
    {
        glm::vec3 normals;
    };

    struct Mesh{
        std::vector<Vertex> coordsV;
        std::vector<TexturesV> textureC;
        std::vector<NormalsV> normalV;
        vector<unsigned int> indices;
    };

    inline int getNumberOfVertices(){
        return numVertices;
    }

    inline int getNumberOfTextures(){
        return numTextures;
    }

/*inline int ModelOBJ::getNumberOfTriangles(){
	return numTriangles;
}*/

    inline int getNumberOfGeometries(){
        return numGeometries;
    }

    inline int getNumberOfMaterials(){
        return numMaterials;
    }

    inline int getNumberOfMeshes(){
        return numMeshes;
    }

    inline int getNumberOfNormals(){
        return numNormals;
    }

    inline float getNumberOfAverage(){
        return average;
    }

    inline float getNumberOfMiddle(){
        return middle;
    }

    inline float getNumberOfsDerivation(){
        return sDeviation;
    }

    inline float getNumberOfAverageX(){
        return averageX;
    }

    inline float getNumberOfMiddleX(){
        return middleX;
    }

    inline float getNumberOfsDerivationX(){
        return sDeviationX;
    }

    inline float getNumberOfAverageY(){
        return averageY;
    }

    inline float getNumberOfMiddleY(){
        return middleY;
    }

    inline float getNumberOfsDerivationY(){
        return sDeviationY;
    }

    inline float getNumberOfAverageZ(){
        return averageZ;
    }

    inline float getNumberOfMiddleZ(){
        return middleZ;
    }

    inline float getNumberOfsDerivationZ(){
        return sDeviationZ;
    }

    inline float getMaxdistance(){
        return maxd;
    }

    inline float getMindistance(){
        return mind;
    }

    inline float getMaxdistanceX(){
        return maxDX;
    }

    inline float getMindistanceX(){
        return minDX;
    }

    inline float getMaxdistanceY(){
        return maxDY;
    }

    inline float getMindistanceY(){
        return minDY;
    }

    inline float getMaxdistanceZ(){
        return maxDY;
    }

    inline float getMindistanceZ(){
        return minDY;
    }

    inline vector<unsigned int> getMortonCodes(){
        return mortonCodes;
    }

    inline vector<UniqueVertex> getUniqueVertices(){
        return uniqueVertices;
    }

    inline vector<Mesh> getListOfMeshes(){
        return listMeshes;
    }

    inline vector<std::vector<float>> getMaxandMinCoords(){
        return maxminCoords;
    }

    inline vector<float> getDistances(){
        return distances;
    }

    inline vector<float> getDistancesX(){
        return distancesX;
    }

    inline vector<float> getDistancesY(){
        return distancesY;
    }
    inline vector<float> getDistancesZ(){
        return distancesZ;
    }
    inline vector<glm::vec3> getCentroids(){
        return centroids;
    }
    inline vector<std::vector<glm::vec3>> getCentroidsByMesh(){
        return centroidsByMesh;
    }
    inline vector<glm::vec2> getEspacialFrecuenciesX(){
        return espacialFreqEX;
    }
    inline vector<glm::vec2> getEspacialFrecuenciesY(){
        return espacialFreqEY;
    }
    inline vector<glm::vec2> getEspacialFrecuenciesZ(){
        return espacialFreqEZ;
    }
    inline vector<glm::vec2> getHistogramFrecuencies(){
        return frequenciesC;
    }
    inline vector<glm::vec2> getHistogramFrecuenciesX(){
        return frequenciesCX;
    }
    inline vector<glm::vec2> getHistogramFrecuenciesY(){
        return frequenciesCY;
    }
    inline vector<glm::vec2> getHistogramFrecuenciesZ(){
        return frequenciesCZ;
    }

    ModelOBJ();
    int import(string path);
    void buildMesh(const aiScene* scene);
    void loadMesh(const aiMesh* mesh);
    void maxMinCoord();
    uint32_t codeMortonCode(uint32_t x, uint32_t y, uint32_t z);
    Vertex decodeMortonCode(unsigned int mortonC);
    void precomputingLUT();
    void clusteringMortonCodes(vector<UniqueVertex> v);
    void vertexSeparationCentroids();
    void vertexSeparationCentroidsX();
    void vertexSeparationCentroidsY();
    void vertexSeparationCentroidsZ();
    void kMeansClustering();
    void cTandsDKmeas();
    void frequencies();
    void frequenciesX();
    void frequenciesY();
    void frequenciesZ();
    void frequenciesEX();
    void frequenciesEY();
    void frequenciesEZ();
    void histogramFrequency();
    void histogramFrequencyX();
    void histogramFrequencyY();
    void histogramFrequencyZ();
    void espacialHistogramsX();
    void espacialHistogramsY();
    void espacialHistogramsZ();
    void centroid();
    void centralTendencyCoordsX();
    void standardDeviationCoordsX();
    void centralTendencyCoordsY();
    void standardDeviationCoordsY();
    void centralTendencyCoordsZ();
    void standardDeviationCoordsZ();
    void centralTendency();
    void standardDeviation();
    void characterizeScene();
    UniqueVertex normalize(UniqueVertex v);
    float maxCoordX = __DBL_MIN__, maxCoordY = __DBL_MIN__, maxCoordZ = __DBL_MIN__,minCoordX = __DBL_MAX__, minCoordY = __DBL_MAX__, minCoordZ = __DBL_MAX__; //Estaba en privado

private:
    int numVertices = 0,numGeometries = 0,numMaterials = 0,numMeshes = 0,numNormals = 0 ,numTextures = 0;
    float average = 0,middle = 0,sDeviation = 0;
    float averageX = 0,middleX = 0,sDeviationX = 0;
    float averageY = 0,middleY = 0,sDeviationY = 0;
    float averageZ = 0,middleZ = 0,sDeviationZ = 0;
    float maxd = 0, mind = 0, maxDX = 0, minDX = 0, maxDY = 0, minDY = 0, maxDZ = 0, minDZ = 0;
    vector<float> coordsX;
    vector<float> coordsY;
    vector<float> coordsZ;
    vector<const aiMesh*> meshes;
    vector<uint32_t> mortonCodes;
    vector<UniqueVertex> uniqueVertices;
    vector<float> averageByCluster;
    vector<float> middleByCluster;
    vector<float> sDerivationByCluster;
    //std::vector<glm::vec3> infoVertexClusters;
    vector<Mesh> listMeshes;
    vector<std::vector<float>> maxminCoords;
    vector<float> distances;
    vector<float> distancesX;
    vector<float> distancesY;
    vector<float> distancesZ;
    vector<int> clusterinepoch;
    vector<glm::vec3> centroids;
    vector<std::vector<glm::vec3>> centroidsByMesh;
    vector<glm::vec2> espacialFreqEX;
    vector<glm::vec2> espacialFreqEY;
    vector<glm::vec2> espacialFreqEZ;
    vector<glm::vec2> frequenciesC;
    vector<glm::vec2> frequenciesCX;
    vector<glm::vec2> frequenciesCY;
    vector<glm::vec2> frequenciesCZ;
};
#endif MODELOBJ_H