#include "bvh.h"
#include "bounds.h"
#include "point.h"
#include "ray.h"
#include "model.h"
#include "utils.h"
#include <iostream>
#include <time.h>
#include <string>

using namespace std;
void parseFile(string &modelName);
int main(){
    // Load model
    string modelName;
    parseFile(modelName);
    auto* obj = new Model(modelName);
    obj->loadModel();
    // Obtaining elements to process
    vector<vector<glm::vec3>> all_centroids = obj->objLoader.getCentroidsByMesh();
    vector<ModelOBJ::Mesh> meshes = obj->objLoader.getListOfMeshes();
    int total_meshes = obj->objLoader.getListOfMeshes().size();
    vector<Ray3f> test_rays;
    Point3f p_min(obj->objLoader.minCoordX, obj->objLoader.minCoordY, obj->objLoader.minCoordZ);
    Point3f p_max(obj->objLoader.maxCoordX, obj->objLoader.maxCoordY, obj->objLoader.maxCoordZ);
    float width = p_max.x - p_min.x;
    float height = p_max.y - p_min.y;
    float const_z = (p_max.z - p_min.z)/2;
    Point3f origin(width/2, height/2, -100);
    bool normalized = true;
    if(normalized)
    {
        for(float w = 0; w < 1; w+=0.1)
        {
            for(float h = 0; h < 1; h+=0.1)
            {
                test_rays.emplace_back(origin, Point3f(w,h,const_z));
            }
        }
    }else{
        if((p_max.x - p_min.x) < 1)
        {
            float w = p_min.x;
            while(w < p_max.x)
            {
                float h = p_min.y;
                while(h < p_max.y)
                {
                    Point3f dir(w,h,const_z);
                    Ray3f temp_ray(origin, dir);
                    test_rays.emplace_back(temp_ray);
                    h+=0.01;
                }
                w+=0.01;
            }
        }else
        {
            for(float w = (p_min.x + 1); w < width; w+=1) // w+=50
            {
                for(float h = (p_min.y + 1); h < height; h+=1) // h+=50
                {
                    test_rays.emplace_back(origin, Point3f(w,h,const_z));
                }
            }
        }
    }
    int total_geometries = obj->objLoader.getNumberOfGeometries();
    int processed_geometries = 0;
    vector<double> unmark_time;
    vector<double> lineal_time;
    cout<<"--------------------------------------------------"<< endl;
    cout<<"RAYOS GENERADOS: "<< test_rays.size() << endl;
    cout<<"--------------------------------------------------"<< endl;
    double construction_average = 0.0;
    double fill_average = 0.0;
    double total_bvh_elapsed = 0.0;
    double total_linear_elapsed = 0.0;
    for(int step = 0; step < total_meshes; step++)
    {
        vector<glm::vec3> current_centroids = all_centroids[step];
        ModelOBJ::Mesh current_mesh = meshes[step];
        auto* accel_struct = new BVH<float>(current_mesh);
        int current_num_cent = current_centroids.size();
        //Testing auto_construct function
        total_geometries -= current_num_cent;
        clock_t t_construct = clock();
        accel_struct->auto_construct(current_num_cent, false);
        clock_t t_cons_end = clock();
        processed_geometries += current_num_cent;
        construction_average+= double(t_cons_end - t_construct)/CLOCKS_PER_SEC;
        //Testing fill_up function
        clock_t t_fill = clock();
        accel_struct->set_scene_extent(p_min, p_max, false);
        accel_struct->fill_up_beta(current_centroids, false);
        //accel_struct->set_scene_extent(p_min, p_max, true);
        clock_t t_fill_end = clock();
        fill_average += double(t_fill_end - t_fill)/CLOCKS_PER_SEC;
        accel_struct->check_nodes();
        //Testing level_order_traversal traversal
        int valid_rays_bvh = 0;
        clock_t t_bvh_traversal = clock();
        for(Ray3f ray: test_rays)
        {
            clock_t t_unmark = clock();
            accel_struct->unmark_nodes();
            clock_t t_unmark_end = clock();
            unmark_time.emplace_back(double(t_unmark_end - t_unmark)/CLOCKS_PER_SEC);
            if(accel_struct->traversal_v2(ray))
            {
                valid_rays_bvh++;
            }
        }
        clock_t t_bvh_end = clock();
        double unmark_elapsed = 0.0;
        for(double t_elapsed: unmark_time)
        {
            unmark_elapsed+=t_elapsed;
        }
        total_bvh_elapsed += double((t_bvh_end - t_bvh_traversal) - unmark_elapsed)/CLOCKS_PER_SEC;
        //Testing linear_traversal
        int valid_rays_linear = 0;
        clock_t t_linear_traversal = clock();
        for(Ray3f l_ray: test_rays)
        {
            if(accel_struct->linear_traversal_beta(l_ray))
            {
                valid_rays_linear++;
            }
        }
        clock_t t_linear_end = clock();
        total_linear_elapsed += double(t_linear_end - t_linear_traversal)/CLOCKS_PER_SEC;
        cout<<"--------------------------------------------------"<< endl;
        cout<< "MESH: " << (step+1) <<endl;
        cout<< valid_rays_bvh <<endl;
        cout<< valid_rays_linear <<endl;
    }
    cout<<"--------------------------------------------------"<< endl;
    cout<<"Tiempo promedio de construccion del arbol: "<< setprecision(12) << construction_average/total_meshes << endl;
    cout<<"Tiempo promedio de llenado del arbol: "<< setprecision(12) << fill_average/total_meshes << endl;
    cout<<"Tiempo de recorrido del arbol: "<< setprecision(12) << total_bvh_elapsed << endl;
    cout<<"Tiempo de recorrido del vector: "<< setprecision(12) << total_linear_elapsed << endl;
    cout<<"--------------------------------------------------"<< endl;
}

void parseFile(string &modelName)
{
    std::ifstream informationFile;
    std::stringstream tmp;
    informationFile.open("../import_geometry/conf.ini");
    char output[100];
    if (informationFile.is_open()) {
        while (!informationFile.eof())
        {
            informationFile >> output;
            if(output[0]=='M'){
                for(int i=6;output[i]!='\0';i++)
                    tmp<<output[i];
            }
        }
        tmp>>modelName;
    }
}