#include <assert.h>
#include <cstdint>
#ifndef _point_h
#define _point_h
/**
 * Implementation of a 3D Point.
 * This class provides an implementation of a 3D Point in C++.
 * @tparam T generic type.
 */
template <typename T> class Point {
    public:
        // Public attributes
        /// The X coord of this point.
        T x;
        /// The Y coord of this point.
        T y;
        /// The Z coord of this point.
        T z;
        /// The longest AXIS
        int l_axis;
        /// Only for centroids: Represents the index of the corresponding face.
        int c_face;
        /// Only for centroids: Morton code representation for this point.
        uint32_t morton_code;
        /**
         * Default constructor of the Point class.
         */
        Point(){
            x = 0.0;
            y = 0.0;
            z = 0.0;
            l_axis = 3;
            c_face = -2;
            morton_code = 0;
        }

        /**
         * Constructor with parameters of the Point class.
         * @param coordX the X coord of this point.
         * @param coordY the Y coord of this point.
         * @param coordZ the z coord of this point.
         */
        Point(T coordX, T coordY, T coordZ, int axis = 0, int face = -2){
            x = coordX;
            y = coordY;
            z = coordZ;
            l_axis = axis;
            c_face = face;
            morton_code = 0;
        }

        /**
         * Method to allow the access to coords like an array.
         * @param i the i-th space of the 'array'.
         * @return the corresponding coord of the i-th space.
         */
        T operator[](int i) const{
            assert(i >= 0 && i <= 2);
            if(i == 0) return x;
            if(i == 1) return y;
            return z;
        }
};
/// Definition of the Point3F (Point<float>) type.
typedef Point<float> Point3f;
#endif