#include "vector.h"
#include "point.h"
#include <cmath>
#ifndef _ray3_h
#define _ray3_h
/**
 * Implementation of a 3D Ray.
 * This class provides an implementation of a 3D Ray in C++.
 * @tparam T generic type.
 */
template<typename T> class Ray3{
    public:
        //Public attributes.
        /// The origin point of this ray.
        Point<T> origin;
        /// The direction of this ray.
        Vector3<T> direction;
        /// The sign of each coordinate of direction.
        int sign[3];

        /**
         * Constructor of the Ray from given point and vector.
         * @param ori the origin of the ray.
         * @param dir the direction of the ray.
         */
        Ray3(const Point<T> ori, const Vector3<T> dir):
            origin(ori), direction(dir){
                sign[0] = (1/direction.x < 0);
                sign[1] = (1/direction.y < 0);
                sign[2] = (1/direction.z < 0);
            }
        
        /**
         * Constructor of the Ray from two given points.
         * @param ori the origin of the ray.
         * @param fin the last point.
         */
        Ray3(const Point<T> ori, const Point<T> fin):
            origin(ori), direction(Vector3<T>(fin[0]-ori[0],
                fin[1]-ori[1], fin[2]-ori[2])){
                sign[0] = (1/direction.x < 0);
                sign[1] = (1/direction.y < 0);
                sign[2] = (1/direction.z < 0);
            }
};
/// Definition of the Ray3f (Ray3<float>) type.
typedef Ray3<float> Ray3f;
#endif 
