#include "point.h"
#include "vector.h"
#include <cmath>
#include <limits>
#include "ModelOBJ.h"
#include <iostream>
#include <algorithm>
#ifndef REAL_TIME_RAYTRACING_SC_UTILS_H
#define REAL_TIME_RAYTRACING_SC_UTILS_H

/**
 * Method that determine the total amount of bits needed for represent the current number of primitives.
 * @param primitives Current number of primitives.
 * @return str_representation Binary representation
 */
string binary_representation(const int &primitives)
{
    int cp_prim = primitives;
    string str_representation = "";
    while(cp_prim)
    {
        str_representation += (cp_prim%2) + '0';
        cp_prim /= 2;
    }
    reverse(str_representation.begin(), str_representation.end());
    return str_representation;
}

/**
 *
 * @param centroids
 * @return axis
 */
int find_longest_axis(const vector<Point3f> &centroids)
{
    float axis = 3;
    float min_X = numeric_limits<float>::max();
    float min_Y = numeric_limits<float>::max();
    float min_Z = numeric_limits<float>::max();
    float max_X = numeric_limits<float>::min();
    float max_Y = numeric_limits<float>::min();
    float max_Z = numeric_limits<float>::min();
    for(Point3f point : centroids)
    {
        min_X = (min_X > point[0])?point[0]:min_X;
        min_Y = (min_Y > point[1])?point[1]:min_Y;
        min_Z = (min_Z > point[2])?point[2]:min_Z;
        max_X = (max_X < point[0])?point[0]:max_X;
        max_Y = (max_Y < point[1])?point[1]:max_Y;
        max_Z = (max_Z < point[2])?point[2]:max_Z;
    }
    float long_X = max_X - min_X;
    float long_Y = max_Y - min_Y;
    float long_Z = max_Z - min_Z;
    axis = max(long_X, max(long_Y, long_Z));
    //Esto se cambio, si se quita funciona pero no deberia
    if(long_X == axis)
        return 0;
    if(long_Y == axis)
        return 1;
    return 2;
}

/**
 *
 * @param p1
 * @param p2
 * @return
 */
bool sort_by_axis(Point3f p1, Point3f p2) { return (p1[p1.l_axis] < p2[p2.l_axis]);}

/**
 *
 * @param actualPoint
 * @param pointMax
 * @param pointMin
 * @return
 */
Point3f normalize(Point3f actualPoint, Point3f pointMax, Point3f pointMin)
{
    Point3f normalized(0.0,0.0,0.0);
    normalized.x = (actualPoint.x - pointMin.x) / (pointMax.x - pointMin.x);
    normalized.y = (actualPoint.y - pointMin.y) / (pointMax.y - pointMin.y);
    normalized.z = (actualPoint.z - pointMin.z) / (pointMax.z - pointMin.z);
    normalized.c_face = actualPoint.c_face;
    return normalized;
}

#endif //REAL_TIME_RAYTRACING_SC_UTILS_H
